import { BridgeSvr } from 'service/bridge.svr'
import { BaseController, role, ROLE_ANON, router, ZError } from 'zutils'

const TOKEN_PREFIX = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.'

class WalletBridgeController extends BaseController {
  @role(ROLE_ANON)
  @router('post /bridge/regist')
  async registWebClient(req, res) {
    let { clientId } = req.params
    if (!clientId) {
      throw new ZError(10, 'params mismatch')
    }
    const qrId = new BridgeSvr().registClientReq(clientId)
    let token = await res.jwtSign({ qrId })
    token = token.replace(TOKEN_PREFIX, '')
    return { token }
  }

  @role(ROLE_ANON)
  @router('post /bridge/status')
  async bridgeStatus(req, res) {
    let { webtoken, clientId } = req.params
    if (!webtoken || !clientId) {
      throw new ZError(10, 'params mismatch')
    }
    const token = webtoken.indexOf(TOKEN_PREFIX) === -1 ? TOKEN_PREFIX + webtoken : webtoken
    const reqData = await req.jwtVerify({ extractToken: () => token })
    if (!reqData || !reqData.qrId) {
      throw new ZError(14, 'token error')
    }
    const { qrId } = reqData
    const data = new BridgeSvr().fetchData(qrId)
    if (!data) {
      throw new ZError(11, 'data not found')
    }
    if (data.clientId !== clientId) {
      throw new ZError(13, 'login info mismatch')
    }
    if (data.status === 0 && data.expire < Date.now()) {
      data.status = -1
    }
    return data
  }
  /**
   * 1. 验证游戏客户端的token
   * 2. 验证扫码获得的web端的token, 获取qrId
   * 3. 生成一个有时效的token, 保存至cache, 供web端获取
   */
  @router('post /bridge/upload')
  async uploadKeyInfo(req, res) {
    let { key, webtoken } = req.params
    let user = req.user
    if (!webtoken || !key) {
      throw new ZError(10, 'params mismatch')
    }
    webtoken = webtoken.indexOf(TOKEN_PREFIX) === -1 ? TOKEN_PREFIX + webtoken : webtoken
    const reqData = await req.jwtVerify({ extractToken: () => webtoken })
    if (!reqData || !reqData.qrId) {
      throw new ZError(14, 'token error')
    }
    const { qrId } = reqData
    const token = await res.jwtSign({ id: user.id })
    let result = new BridgeSvr().updateData(qrId, key, token)
    return { result }
  }
}
