import { getModelForClass, index, modelOptions, mongoose, prop, ReturnModelType, Severity } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'
import { IPayResult } from 'service/game.svr'

/**
 * 上报游戏服的失败记录
 * 定时从这表里获取数据, 上报给游戏服, 上报成功后删除
 */
@dbconn('pay')
@index({ channel: 1, recordId: 1, status: 1 }, { unique: true })
@index({ orderId: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'report_queue', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
export class ReportQueueClass extends BaseModule {
  // 渠道: alchemy, google, apple
  @prop()
  public channel: string
  // 渠道返回的订单id
  @prop()
  public recordId: string
  // 游戏服订单id
  @prop()
  public orderId: string
  // 根据不同渠道有不同含义
  // apple, google: productid
  // alchemy: account_id
  @prop()
  public dataId: string
  // 只有alchemy有, 交易的hash
  @prop()
  public txhash: string
  // 订单状态
  @prop()
  public status: number
  // 上报状态
  @prop({ default: 0 })
  public reportStatus: number

  @prop()
  @prop({ type: () => [mongoose.Schema.Types.Mixed] })
  public reps: any[]

  public toInAppData(): IPayResult {
    return {
      productId: this.dataId,
      gameOrderId: this.orderId,
      orderId: this.recordId,
      status: this.status,
    }
  }

  public toPayData() {
    return {
      gameAccountId: this.dataId,
      gameOrderId: this.orderId,
      status: this.status,
      id: this.recordId,
      txhash: this.txhash || '',
    }
  }

  public static async findUnfinishedRecords(this: ReturnModelType<typeof ReportQueueClass>) {
    return this.find({ reportStatus: 0 }).exec()
  }
}

export const ReportQueue = getModelForClass(ReportQueueClass, { existingConnection: ReportQueueClass.db })
