import { getModelForClass, index, modelOptions, mongoose, prop, Severity } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

@dbconn()
@index({ account: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'wallet', timestamps: true },
})
class WalletClass extends BaseModule {
  @prop({ required: true })
  public account!: string

  /**
   * 钱包的master key
   */
  @prop()
  public key: string

  @prop()
  public address: string
  /**
   * 用于客户端生成密钥时的加盐
   */
  @prop()
  public salt: string

  @prop({ required: true, default: true })
  public nweRecord: boolean

  public toJson() {
    return {
      key: this.key,
      address: this.address,
      salt: this.salt,
    }
  }
}

export const Wallet = getModelForClass(WalletClass, { existingConnection: WalletClass.db })
