import axios from 'axios'

import { queryPrice } from './alchemy.svr'
import * as schedule from 'node-schedule'
import logger from 'logger/logger'
import { erc20TransferGas, ethTransferGas } from './chain.svr'
import { singleton } from 'zutils'

@singleton
export class GasSvr {
  private priceMap: Map<string, number> = new Map()

  public async estimateGas(data: any) {
    let { crypto, network } = data
    network = network || 'arbitrum'
    let key = `${crypto}_${network}`
    let price = this.priceMap.get(key)
    if (!price) {
      await this.refreshOne({ crypto, network })
      price = this.priceMap.get(key)
    }
    return price
  }

  private async refreshOne(data) {
    let { crypto, network } = data
    let key = `${crypto}_${network}`
    try {
      let res
      if (crypto.toLowerCase() === 'ceg') {
        res = await erc20TransferGas()
      } else if (crypto.toLowerCase() === 'eth' || crypto.toLowerCase() === 'agor') {
        res = await ethTransferGas()
      }
      let priceData = res.data
      if (!priceData.error) {
        let gas = ((priceData.result | 1) * +process.env.CHAIN_GAS_BOOST) | 1
        this.priceMap.set(key, gas)
      }
    } catch (e) {
      logger.debug('update eth gas error: ' + e.message || e)
    }
  }

  private async refreshAll() {
    for (let key of this.priceMap.keys()) {
      let [crypto, chain] = key.split('_')
      let data = {
        crypto,
        network: chain,
      }
      await this.refreshOne(data)
    }
  }

  scheduleAll() {
    const job = schedule.scheduleJob('*/30 * * * * *', async () => {
      this.refreshAll()
    })
  }
}
