import { androidpublisher_v3, google } from 'googleapis'
import logger from 'logger/logger'
import { singleton } from 'zutils'

const PAGEAGE_NAME = 'com.cege.games.release'

@singleton
export class GooglePaySvr {
  private androidpublisher: androidpublisher_v3.Androidpublisher
  constructor() {}

  public async init() {
    // this.androidpublisher = google.androidpublisher('v3')
    const auth = new google.auth.GoogleAuth({
      keyFile: 'google_cloud.json',
      // Scopes can be specified either as an array or as a single, space-delimited string.
      scopes: ['https://www.googleapis.com/auth/androidpublisher'],
    })
    this.androidpublisher = google.androidpublisher({
      version: 'v3',
      auth,
    })
  }

  public async fetchPurchaseData(productId: string, purchaseToken: string) {
    logger.info(`fetchPurchaseData::productId=${productId}, purchaseToken=${purchaseToken}`)
    return this.androidpublisher.purchases.products.get({
      packageName: PAGEAGE_NAME,
      productId: productId,
      token: purchaseToken,
    })
  }

  public async consumePurchase(productId: string, purchaseToken: string) {
    return this.androidpublisher.purchases.products.consume({
      packageName: PAGEAGE_NAME,
      productId: productId,
      token: purchaseToken,
    })
  }
  /**
   * 查询作废的交易
   * @param startTime
   * @param endTime
   */
  public async queryVoidedPurchases(startTime: number, endTime: number) {
    return this.androidpublisher.purchases.voidedpurchases.list({
      packageName: PAGEAGE_NAME,
      type: 0,
      startTime: startTime + '',
      endTime: endTime + '',
    })
  }
}
