import { HTTP_GET, HTTP_POST } from 'common/Constants'
import crypto from 'crypto'

const apiKey = process.env.OKX_API_KEY
const projectId = process.env.OKX_PROJECT_ID
const pass = process.env.OKX_PASS
const secretKey = process.env.OKX_SECRET_KEY

export function prepareOkxReqCfg(config: any) {
  let timestamp = new Date().toISOString()
  let url = new URL(config.url)
  let { method } = config.method
  let signStr = timestamp + method + url.pathname
  if (method === HTTP_GET) {
    signStr += url.search
  } else if (method === HTTP_POST) {
    let bodyStr = JSON.stringify(JSON.parse(config.data))
    signStr += bodyStr
  }
  const mac = crypto.createHmac('sha256', secretKey)
  const sign = mac.update(signStr).digest('base64')
  config.headers = {
    'OK-ACCESS-PROJECT': projectId,
    'OK-ACCESS-KEY': apiKey,
    'OK-ACCESS-SIGN': sign,
    'OK-ACCESS-TIMESTAMP': timestamp,
    'OK-ACCESS-PASSPHRASE': pass,
    'Content-Type': 'application/json',
  }
  return config
}
